@@ -333,6 +333,15 @@ Useful for further Tedana processing post-NiBabies.""",
         help="Replace medial wall values with NaNs on functional GIFTI files. Only "
         "performed for GIFTI files mapped to a freesurfer subject (fsaverage or fsnative).",
     )
+    g_conf.add_argument(
+        "--project-goodvoxels",
+        required=False,
+        action="store_true",
+        default=False,
+        help="Exclude voxels whose timeseries have locally high coefficient of variation "
+        "from surface resampling. Only performed for GIFTI files mapped to a freesurfer subject "
+        "(fsaverage or fsnative).",
+    )
     g_conf.add_argument(
         "--slice-time-ref",
         required=False,
@@ -450,12 +459,6 @@ Useful for further Tedana processing post-NiBabies.""",
         default=True,
         help="do not remove median (within mask) from fieldmap",
     )
-    g_fmap.add_argument(
-        "--topup-max-vols",
-        default=5,
-        type=int,
-        help="maximum number of volumes to use with TOPUP, per-series (EPI or BOLD)",
-    )
 
     # SyN-unwarp options
     g_syn = parser.add_argument_group("Specific options for SyN distortion correction")
@@ -661,6 +664,12 @@ discourage its usage."""
         action="store_true",
         help="Force traditional FreeSurfer surface reconstruction.",
     )
+    g_baby.add_argument(
+        "--surface-recon-method",
+        choices=("infantfs", "freesurfer", "mcribs"),
+        default="infantfs",
+        help="Method to use for surface reconstruction",
+    )
     return parser
 
 
@@ -671,6 +680,14 @@ def parse_args(args=None, namespace=None):
     parser = _build_parser()
     opts = parser.parse_args(args, namespace)
 
+    # Deprecations
+    if opts.force_reconall:
+        config.loggers.cli.warning(
+            "--force-reconall is deprecated and will be removed in a future release."
+            "To run traditional `recon-all`, use `--surface-recon-method freesurfer` instead."
+        )
+        opts.surface_recon_method = "freesurfer"
+
     if opts.config_file:
         skip = {} if opts.reports_only else {"execution": ("run_uuid",)}
         config.load(opts.config_file, skip=skip)
@@ -679,20 +696,6 @@ def parse_args(args=None, namespace=None):
     config.execution.log_level = int(max(25 - 5 * opts.verbose_count, logging.DEBUG))
     config.from_dict(vars(opts))
 
-    # Initialize --output-spaces if not defined
-    if config.execution.output_spaces is None:
-        from niworkflows.utils.spaces import Reference, SpatialReferences
-
-        from ..utils.misc import cohort_by_months
-
-        if config.workflow.age_months is None:
-            parser.error("--age-months must be provided if --output-spaces is not set.")
-
-        cohort = cohort_by_months("MNIInfant", config.workflow.age_months)
-        config.execution.output_spaces = SpatialReferences(
-            [Reference("MNIInfant", {"res": "native", "cohort": cohort})]
-        )
-
     # Retrieve logging level
     build_log = config.loggers.cli
 
@@ -742,14 +745,21 @@ applied."""
 
     if config.execution.fs_subjects_dir is None:
         if output_layout == "bids":
-            config.execution.fs_subjects_dir = output_dir / "sourcedata" / "infant-freesurfer"
+            config.execution.fs_subjects_dir = output_dir / "sourcedata" / "freesurfer"
         elif output_layout == "legacy":
-            config.execution.fs_subjects_dir = output_dir / "infant-freesurfer"
+            config.execution.fs_subjects_dir = output_dir / "freesurfer"
     if config.execution.nibabies_dir is None:
         if output_layout == "bids":
             config.execution.nibabies_dir = output_dir
         elif output_layout == "legacy":
             config.execution.nibabies_dir = output_dir / "nibabies"
+    if config.workflow.surface_recon_method == "mcribs":
+        if output_layout == "bids":
+            config.execution.mcribs_dir = output_dir / "sourcedata" / "mcribs"
+        elif output_layout == "legacy":
+            config.execution.mcribs_dir = output_dir / "mcribs"
+        # Ensure the directory is created
+        config.execution.mcribs_dir.mkdir(exist_ok=True, parents=True)
 
     # Wipe out existing work_dir
     if opts.clean_workdir and work_dir.exists():
@@ -807,8 +817,41 @@ applied."""
 
     config.execution.participant_label = sorted(participant_label)
     config.workflow.skull_strip_template = config.workflow.skull_strip_template[0]
+    config.execution.unique_labels = compute_subworkflows()
 
     # finally, write config to file
     config_file = config.execution.work_dir / config.execution.run_uuid / "config.toml"
     config_file.parent.mkdir(exist_ok=True, parents=True)
     config.to_filename(config_file)
+
+
+def compute_subworkflows() -> list:
+    """
+    Query all available participants and sessions, and construct the combinations of the
+    subworkflows needed.
+    """
+    from niworkflows.utils.bids import collect_participants
+
+    from nibabies import config
+
+    # consists of (subject_id, session_id) tuples
+    subworkflows = []
+
+    subject_list = collect_participants(
+        config.execution.layout,
+        participant_label=config.execution.participant_label,
+        strict=True,
+    )
+
+    for subject in subject_list:
+        # Due to rapidly changing morphometry of the population
+        # Ensure each subject session is processed individually
+        sessions = (
+            config.execution.session_id
+            or config.execution.layout.get_sessions(scope='raw', subject=subject)
+            or [None]
+        )
+        # grab participant age per session
+        for session in sessions:
+            subworkflows.append((subject, session))
+    return subworkflows
