@@ -169,13 +169,13 @@ class CiftiCreateDenseFromTemplate(WBCommand):
     >>> frmtpl.inputs.series = True
     >>> frmtpl.inputs.series_step = 0.8
     >>> frmtpl.inputs.series_start = 0
-    >>> frmtpl.cmdline
+    >>> frmtpl.cmdline  # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
     'wb_command -cifti-create-dense-from-template .../func.dtseries.nii \
     template_func.dtseries.nii -series 0.8 0.0'
 
     >>> frmtpl.inputs.volume = [("OTHER", data_dir / 'functional.nii', True), \
         ("PUTAMEN_LEFT", data_dir / 'functional.nii')]
-    >>> frmtpl.cmdline
+    >>> frmtpl.cmdline  # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
     'wb_command -cifti-create-dense-from-template .../func.dtseries.nii \
     template_func.dtseries.nii -series 0.8 0.0 \
     -volume OTHER .../functional.nii -from-cropped \
@@ -333,7 +333,7 @@ class CiftiCreateDenseTimeseries(WBCommand):
     >>> createdts = CiftiCreateDenseTimeseries()
     >>> createdts.inputs.volume_data = data_dir /'functional.nii'
     >>> createdts.inputs.volume_structure_labels = data_dir / 'atlas.nii'
-    >>> createdts.cmdline
+    >>> createdts.cmdline  # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
     'wb_command -cifti-create-dense-timeseries out.dtseries.nii \
     -volume .../functional.nii .../atlas.nii'
     """
@@ -470,7 +470,7 @@ class CiftiCreateLabel(WBCommand):
     >>> lab = wb.CiftiCreateLabel()
     >>> lab.inputs.volume_label = data_dir / "functional.nii"
     >>> lab.inputs.structure_label_volume = data_dir / "functional.nii"
-    >>> lab.cmdline
+    >>> lab.cmdline # doctest: +ELLIPSIS
     'wb_command -cifti-create-label out.dlabel.nii -volume .../functional.nii .../functional.nii'
     """
 
@@ -949,7 +949,7 @@ class CiftiResample(WBCommand):
     >>> res.inputs.volume_method = "CUBIC"
     >>> res.inputs.out_file = "resampled.dtseries.nii"
     >>> res.inputs.volume_predilate = 10
-    >>> res.cmdline
+    >>> res.cmdline  # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
     'wb_command -cifti-resample .../func.dtseries.nii COLUMN .../func.dlabel.nii COLUMN \
     ADAP_BARY_AREA CUBIC resampled.dtseries.nii -volume-predilate 10'
     """
@@ -1055,7 +1055,7 @@ class CiftiSeparate(WBCommand):
     >>> separate.inputs.in_file = data_dir / "func.dtseries.nii"
     >>> separate.inputs.direction = "COLUMN"
     >>> separate.inputs.volume_all_file = "volume_all.nii.gz"
-    >>> separate.cmdline
+    >>> separate.cmdline  # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
     'wb_command -cifti-separate .../func.dtseries.nii COLUMN \
     -volume-all volume_all.nii.gz'
 
@@ -1063,7 +1063,7 @@ class CiftiSeparate(WBCommand):
     >>> separate.inputs.metric = [("CORTEX_LEFT", "cortexleft.func.gii")]
     >>> separate.inputs.volume = [("HIPPOCAMPUS_LEFT", "hippoL.nii.gz"), \
         ("HIPPOCAMPUS_RIGHT", "hippoR.nii.gz", "hippoR.roi.nii.gz")]
-    >>> separate.cmdline
+    >>> separate.cmdline  # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
     'wb_command -cifti-separate .../func.dtseries.nii COLUMN \
     -volume-all volume_all.nii.gz -metric CORTEX_LEFT cortexleft.func.gii \
     -volume HIPPOCAMPUS_LEFT hippoL.nii.gz \
@@ -1221,7 +1221,7 @@ class VolumeAffineResample(WBCommand):
     >>> resample.inputs.volume_space = data_dir /'anatomical.nii'
     >>> resample.inputs.method = 'CUBIC'
     >>> resample.inputs.affine = data_dir / 'func_to_struct.mat'
-    >>> resample.cmdline
+    >>> resample.cmdline  # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
     'wb_command -volume-resample .../functional.nii .../anatomical.nii CUBIC \
     resampled_functional.nii.gz -affine .../func_to_struct.mat'
 
@@ -1230,7 +1230,7 @@ class VolumeAffineResample(WBCommand):
     for references.
 
     >>> resample.inputs.flirt = True
-    >>> resample.cmdline
+    >>> resample.cmdline  # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
     'wb_command -volume-resample .../functional.nii .../anatomical.nii CUBIC \
     resampled_functional.nii.gz -affine .../func_to_struct.mat \
     -flirt .../functional.nii .../anatomical.nii'
@@ -1240,7 +1240,7 @@ class VolumeAffineResample(WBCommand):
 
     >>> resample.inputs.flirt_source_volume = data_dir / 'epi.nii'
     >>> resample.inputs.flirt_target_volume = data_dir /'T1w.nii'
-    >>> resample.cmdline
+    >>> resample.cmdline  # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
     'wb_command -volume-resample .../functional.nii .../anatomical.nii CUBIC \
     resampled_functional.nii.gz -affine .../func_to_struct.mat \
     -flirt .../epi.nii .../T1w.nii'
@@ -1300,7 +1300,7 @@ class VolumeAllLabelsToROIs(WBCommand):
     >>> rois = VolumeAllLabelsToROIs()
     >>> rois.inputs.in_file = data_dir / 'atlas.nii'
     >>> rois.inputs.label_map = 1
-    >>> rois.cmdline
+    >>> rois.cmdline  # doctest: +ELLIPSIS
     'wb_command -volume-all-labels-to-rois .../atlas.nii 1 atlas_rois.nii.gz'
     """
 
@@ -1349,7 +1349,7 @@ class VolumeLabelExportTable(WBCommand):
     >>> label_export = VolumeLabelExportTable()
     >>> label_export.inputs.in_file = data_dir / 'atlas.nii'
     >>> label_export.inputs.label_map = 1
-    >>> label_export.cmdline
+    >>> label_export.cmdline  # doctest: +ELLIPSIS
     'wb_command -volume-label-export-table .../atlas.nii 1 atlas_labels.txt'
     """
 
@@ -1437,7 +1437,7 @@ class VolumeLabelImport(WBCommand):
     >>> label_import = VolumeLabelImport()
     >>> label_import.inputs.in_file = data_dir / 'atlas.nii'
     >>> label_import.inputs.label_list_file = data_dir / 'label_list.txt'
-    >>> label_import.cmdline
+    >>> label_import.cmdline  # doctest: +ELLIPSIS +NORMALIZE_WHITESPACE
     'wb_command -volume-label-import .../atlas.nii .../label_list.txt \
     atlas_labels.nii.gz'
     """
@@ -1445,3 +1445,238 @@ class VolumeLabelImport(WBCommand):
     input_spec = VolumeLabelImportInputSpec
     output_spec = VolumeLabelImportOutputSpec
     _cmd = "wb_command -volume-label-import"
+
+
+class CreateSignedDistanceVolumeInputSpec(CommandLineInputSpec):
+    surf_file = File(
+        exists=True,
+        mandatory=True,
+        argstr="%s",
+        position=0,
+        desc="Input surface GIFTI file (.surf.gii)",
+    )
+    ref_file = File(
+        exists=True,
+        mandatory=True,
+        argstr="%s",
+        position=1,
+        desc="NIfTI volume in the desired output space (dims, spacing, origin)",
+    )
+    out_file = File(
+        name_source=["surf_file"],
+        name_template="%s_distvol.nii.gz",
+        argstr="%s",
+        position=2,
+        desc="Name of output volume containing signed distances",
+    )
+    out_mask = File(
+        name_source=["surf_file"],
+        name_template="%s_distmask.nii.gz",
+        argstr="-roi-out %s",
+        desc="Name of file to store a mask where the ``out_file`` has a computed value",
+    )
+    fill_value = traits.Float(
+        0.0,
+        mandatory=False,
+        usedefault=True,
+        argstr="-fill-value %f",
+        desc="value to put in all voxels that don't get assigned a distance",
+    )
+    exact_limit = traits.Float(
+        5.0,
+        usedefault=True,
+        argstr="-exact-limit %f",
+        desc="distance for exact output in mm",
+    )
+    approx_limit = traits.Float(
+        20.0,
+        usedefault=True,
+        argstr="-approx-limit %f",
+        desc="distance for approximate output in mm",
+    )
+    approx_neighborhood = traits.Int(
+        2,
+        usedefault=True,
+        argstr="-approx-neighborhood %d",
+        desc="size of neighborhood cube measured from center to face in voxels, default 2 = 5x5x5",
+    )
+    winding_method = traits.Enum(
+        "EVEN_ODD",
+        "NEGATIVE",
+        "NONZERO",
+        "NORMALS",
+        argstr="-winding %s",
+        usedefault=True,
+        desc="winding method for point inside surface test",
+    )
+
+
+class CreateSignedDistanceVolumeOutputSpec(TraitedSpec):
+    out_file = File(desc="Name of output volume containing signed distances")
+    out_mask = File(
+        desc="Name of file to store a mask where the ``out_file`` has a computed value"
+    )
+
+
+class CreateSignedDistanceVolume(WBCommand):
+    """CREATE SIGNED DISTANCE VOLUME FROM SURFACE
+    wb_command -create-signed-distance-volume
+       <surface> - the input surface
+       <refspace> - a volume in the desired output space (dims, spacing, origin)
+       <outvol> - output - the output volume
+
+       [-roi-out] - output an roi volume of where the output has a computed
+          value
+          <roi-vol> - output - the output roi volume
+
+       [-fill-value] - specify a value to put in all voxels that don't get
+          assigned a distance
+          <value> - value to fill with (default 0)
+
+       [-exact-limit] - specify distance for exact output
+          <dist> - distance in mm (default 5)
+
+       [-approx-limit] - specify distance for approximate output
+          <dist> - distance in mm (default 20)
+
+       [-approx-neighborhood] - voxel neighborhood for approximate calculation
+          <num> - size of neighborhood cube measured from center to face, in
+             voxels (default 2 = 5x5x5)
+
+       [-winding] - winding method for point inside surface test
+          <method> - name of the method (default EVEN_ODD)
+
+       Computes the signed distance function of the surface.  Exact distance is
+       calculated by finding the closest point on any surface triangle to the
+       center of the voxel.  Approximate distance is calculated starting with
+       these distances, using dijkstra's method with a neighborhood of voxels.
+       Specifying too small of an exact distance may produce unexpected results.
+       Valid specifiers for winding methods are as follows:
+
+       EVEN_ODD (default)
+       NEGATIVE
+       NONZERO
+       NORMALS
+
+       The NORMALS method uses the normals of triangles and edges, or the
+       closest triangle hit by a ray from the point.  This method may be
+       slightly faster, but is only reliable for a closed surface that does not
+       cross through itself.  All other methods count entry (positive) and exit
+       (negative) crossings of a vertical ray from the point, then counts as
+       inside if the total is odd, negative, or nonzero, respectively.
+
+    """
+
+    input_spec = CreateSignedDistanceVolumeInputSpec
+    output_spec = CreateSignedDistanceVolumeOutputSpec
+    _cmd = "wb_command -create-signed-distance-volume"
+
+
+class SurfaceAverageInputSpec(CommandLineInputSpec):
+    out_file = File(
+        name_template="averaged.surf.gii",
+        position=0,
+        desc="output file",
+    )
+    surfaces = InputMultiObject(
+        traits.Either(
+            File(exists=True),
+            traits.Tuple(File(exists=True), traits.Float),
+        ),
+        argstr="%s",
+        position=3,
+        desc="Surface, or surface and weighted average tuple, to include in the average",
+    )
+
+
+class SurfaceAverageOutputSpec(TraitedSpec):
+    out_file = File(desc="The output averaged surface")
+    # stddev_metric = File(desc="The output metric for 3D sample standard deviation")
+    # uncert_metric = File(desc="The output metric for uncertainty")
+
+
+class SurfaceAverage(WBCommand):
+    """
+    AVERAGE SURFACE FILES TOGETHER
+    wb_command -surface-average
+      <surface-out> - output - the output averaged surface
+
+      [-stddev] - compute 3D sample standard deviation
+         <stddev-metric-out> - output - the output metric for 3D sample
+            standard deviation
+
+      [-uncertainty] - compute caret5 'uncertainty'
+         <uncert-metric-out> - output - the output metric for uncertainty
+
+      [-surf] - repeatable - specify a surface to include in the average
+         <surface> - a surface file to average
+
+         [-weight] - specify a weighted average
+            <weight> - the weight to use (default 1)
+
+      The 3D sample standard deviation is computed as
+      'sqrt(sum(squaredlength(xyz - mean(xyz)))/(n - 1))'.
+
+      Uncertainty is a legacy measure used in caret5, and is computed as
+      'sum(length(xyz - mean(xyz)))/n'.
+
+      When weights are used, the 3D sample standard deviation treats them as
+      reliability weights.
+    """
+
+    input_spec = SurfaceAverageInputSpec
+    output_spec = SurfaceAverageOutputSpec
+    _cmd = "wb_command -surface-average"
+
+    def _format_arg(self, name, trait_spec, value):
+        if name == 'surfaces':
+            cmd = []
+            for val in value:
+                if len(val) == 2:
+                    cmd.append(f"{val[0]} -weight {val[-1]}")
+                else:
+                    cmd.append(val)
+            return '-surf ' + ' -surf '.join(cmd)
+        return super()._format_arg(name, trait_spec, value)
+
+    def _list_output(self):
+        outputs = self.output_spec().get()
+        outputs["out_file"] = os.path.abspath(self.inputs.out_file)
+        return outputs
+
+
+class SurfaceVertexAreasInputSpec(CommandLineInputSpec):
+    in_file = File(
+        exists=True,
+        mandatory=True,
+        position=0,
+        argstr="%s",
+        desc="Input surface",
+    )
+    out_file = File(
+        name_template="%s.shape.gii",
+        name_source="in_file",
+        position=1,
+        argstr="%s",
+        desc="Output vertex areas",
+    )
+
+
+class SurfaceVertexAreasOutputSpec(TraitedSpec):
+    out_file = File(desc="Output vertex areas")
+
+
+class SurfaceVertexAreas(WBCommand):
+    """
+    MEASURE SURFACE AREA EACH VERTEX IS RESPONSIBLE FOR
+    wb_command -surface-vertex-areas
+      <surface> - the surface to measure
+      <metric> - output - the output metric
+
+      Each vertex gets one third of the area of each triangle it is a part of.
+      Units are mm^2.
+    """
+
+    input_spec = SurfaceVertexAreasInputSpec
+    output_spec = SurfaceVertexAreasOutputSpec
+    _cmd = "wb_command -surface-vertex-areas"
