@@ -1,30 +1,134 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
 """Utilities to handle BIDS inputs."""
+from __future__ import annotations
+
 import json
 import os
 import sys
-from dataclasses import dataclass, field
+import typing as ty
+import warnings
 from pathlib import Path
-from typing import IO, List, Union
+
+import nibabel as nb
+import numpy as np
+from bids.layout import BIDSLayout, Query
+
+_spec: dict = {
+    't1w_mask': {
+        'datatype': 'anat',
+        'desc': 'brain',
+        'space': 'T1w',
+        'suffix': 'mask',
+    },
+    't1w_aseg': {'datatype': 'anat', 'desc': 'aseg', 'space': 'T1w', 'suffix': 'dseg'},
+    't2w_mask': {
+        'datatype': 'anat',
+        'desc': 'brain',
+        'space': 'T2w',
+        'suffix': 'mask',
+    },
+    't2w_aseg': {
+        'datatype': 'anat',
+        'desc': 'aseg',
+        'space': 'T2w',
+        'suffix': 'dseg',
+    },
+}
 
 
-@dataclass
-class BOLDGrouping:
-    """This class is used to facilitate the grouping of BOLD series."""
+class Derivatives:
+    """
+    A container class for collecting and storing derivatives.
 
-    session: Union[str, None]
-    pe_dir: str
-    readout: float
-    multiecho_id: str = None
-    files: List[IO] = field(default_factory=list)
+    A specification (either dictionary or JSON file) can be used to customize derivatives and
+    queries.
+    To populate this class with derivatives, the `populate()` method must first be called.
+    """
+
+    def __getattribute__(self, attr):
+        """In cases where the spec may change, avoid errors."""
+        try:
+            return object.__getattribute__(self, attr)
+        except AttributeError:
+            return None
+
+    def __init__(self, bids_root: Path | str, spec: dict | Path | str | None = None, **args):
+        self.bids_root = Path(bids_root)
+        self.spec = _spec
+        if spec is not None:
+            if not isinstance(spec, dict):
+                spec: dict = json.loads(Path(spec).read_text())
+            self.spec = spec
+
+        self.names = set(self.spec.keys())
+        self.references = {name: None for name in self.names}
+        for name in self.names:
+            setattr(self, name, None)
+
+    def __repr__(self):
+        return '\n'.join([name for name in self.names if getattr(self, name)])
+
+    def __contains__(self, val: str):
+        return val in self.names
+
+    def __bool__(self):
+        return any(getattr(self, name) for name in self.names)
+
+    def populate(
+        self, deriv_path, subject_id: str, session_id: str | Query | None = Query.OPTIONAL
+    ) -> None:
+        """Query a derivatives directory and populate values and references based on the spec."""
+        layout = BIDSLayout(deriv_path, validate=False)
+        for name, query in self.spec.items():
+            items = layout.get(
+                subject=subject_id,
+                session=session_id,
+                extension=['.nii', '.nii.gz'],
+                **query,
+            )
+            if not items or len(items) > 1:
+                warnings.warn(f"Could not find {name}")
+                continue
+            item = items[0]
+
+            # Skip if derivative does not have valid metadata
+            metadata = item.get_metadata()
+            if not metadata or not (reference := metadata.get('SpatialReference')):
+                warnings.warn(f"No metadata found for {item}")
+                continue
+            if isinstance(reference, list):
+                if len(reference) > 1:
+                    warnings.warn(f"Multiple reference found: {reference}")
+                    continue
+                reference = reference[0]
+
+            reference = self.bids_root / reference
+            if not self.validate(item.path, str(reference)):
+                warnings.warn(f"Validation failed between: {item.path} and {reference}")
+                continue
+
+            setattr(self, name, Path(item.path))
+            self.references[name] = reference
 
     @property
-    def name(self) -> str:
-        return f"{self.session}-{self.pe_dir}-{self.readout}-{self.multiecho_id}"
+    def mask(self) -> str | None:
+        return self.t1w_mask or self.t2w_mask
 
-    def add_file(self, fl) -> None:
-        self.files.append(fl)
+    @property
+    def aseg(self) -> str | None:
+        return self.t1w_aseg or self.t2w_aseg
+
+    @staticmethod
+    def validate(derivative: str, reference: str, atol: float = 1e-5) -> bool:
+        anat = nb.load(reference)
+        expected_ort = nb.aff2axcodes(anat.affine)
+        img = nb.load(derivative)
+        if nb.aff2axcodes(img.affine) != expected_ort:
+            return False
+        if img.shape != anat.shape or not np.allclose(anat.affine, img.affine, atol=atol):
+            return False
+        return True
 
 
 def write_bidsignore(deriv_dir):
@@ -127,111 +231,6 @@ def extract_entities(file_list):
     return {k: _unique(v) for k, v in entities.items()}
 
 
-def group_bolds_ref(*, layout, subject, sessions=None):
-    """
-    Extracts BOLD files from a BIDS dataset and combines them into buckets.
-    Files in a bucket share:
-    1) Session
-    2) Phase-encoding direction (PEdir)
-    3) Total readout time (TRT)
-
-    Any files with missing data for (2) or (3) are put in their own bucket.
-
-    Parameters
-    ----------
-    layout : pybids.layout.BIDSLayout
-        Initialized BIDSLayout
-    subject : str
-        The subject ID
-    sessions : None
-
-    Outputs
-    -------
-    combinations : list of tuples
-        Each tuple is composed of (session, PEdir, TRT)
-    files : list of lists
-        Files matching each combination.
-
-    Limitations
-    -----------
-    Single-band reference (sbref) are excluded.
-    """
-    import re
-    from contextlib import suppress
-    from itertools import product
-
-    from sdcflows.utils.epimanip import get_trt
-
-    base_entities = {
-        "subject": subject,
-        "extension": (".nii", ".nii.gz"),
-        "scope": "raw",  # Ensure derivatives are not captured
-    }
-    # dictionary containing unique Groupings and files
-    groupings = {}
-    # list of all BOLDS encountered
-    all_bolds = []
-
-    sessions = sessions if sessions else layout.get_sessions(subject=subject, scope="raw")
-
-    for ses, suffix in sorted(product(sessions or (None,), {"bold"})):
-        # bold files same session
-        bolds = layout.get(suffix=suffix, session=ses, **base_entities)
-        # some sessions may not have BOLD scans
-        if bolds is None:
-            continue
-
-        for i, bold in enumerate(bolds):
-            multiecho_id = None
-            # multi-echo should be grouped together
-            if 'echo' in bold.entities:
-                # create unique id by dropping "_echo-{i}"
-                multiecho_id = re.sub(r"_echo-\d+", "", bold.filename)
-
-            # session, pe, ro
-            meta = bold.get_metadata()
-            pe_dir = meta.get("PhaseEncodingDirection")
-
-            ro = None
-            with suppress(ValueError):
-                ro = get_trt(meta, bold.path)
-            if ro is not None:
-                meta.update({"TotalReadoutTime": ro})
-
-            grouping = BOLDGrouping(
-                session=ses,
-                pe_dir=pe_dir,
-                readout=ro,
-                multiecho_id=multiecho_id,
-            )
-
-            if any(v is None for v in (pe_dir, ro)):
-                # cannot be certain so treat as unique
-                grouping.add_file(bold.path)
-                groupings[f'unknown{i}'] = grouping
-            else:
-                try:
-                    grouping = groupings[grouping.name]
-                except KeyError:
-                    groupings[grouping.name] = grouping
-
-                grouping.add_file(bold.path)
-
-        all_bolds += bolds
-
-        if len(all_bolds) != sum([len(g.files) for _, g in groupings.items()]):
-            msg = f"""Error encountered when grouping BOLD runs.
-Combinations: {groupings}
-BOLD files: {bolds}
-
-Please file a bug-report with the nibabies developers at:
-https://github.com/nipreps/nibabies/issues/new/choose
-"""
-            raise RuntimeError(msg)
-
-    return groupings
-
-
 def validate_input_dir(exec_env, bids_dir, participant_label):
     # Ignore issues and warnings that should not influence NiBabies
     import subprocess
@@ -282,7 +281,7 @@ def validate_input_dir(exec_env, bids_dir, participant_label):
             "MISSING_TSV_COLUMN_EEG_ELECTRODES",
             "MISSING_SESSION",
         ],
-        "error": ["NO_T1W"],
+        "error": [],
         "ignoredFiles": ["/dataset_description.json", "/participants.tsv"],
     }
     # Limit validation only to data from requested participants
@@ -325,45 +324,71 @@ def validate_input_dir(exec_env, bids_dir, participant_label):
             print("bids-validator does not appear to be installed", file=sys.stderr)
 
 
-def collect_precomputed_derivatives(layout, subject_id, derivatives_filters=None):
+def parse_bids_for_age_months(
+    bids_root: str | Path,
+    subject_id: str,
+    session_id: str | None = None,
+) -> int | None:
     """
-    Query and collect precomputed derivatives.
+    Given a BIDS root, query the BIDS metadata files for participant age, in months.
 
-    This function is used to determine which workflow steps can be skipped,
-    based on the files found.
+    The heuristic followed is:
+    1) Check `sub-<subject_id>/sub-<subject_id>_sessions.tsv`
+    2) Check `<root>/participants.tsv`
     """
+    age = None
+    if subject_id.startswith('sub-'):
+        subject_id = subject_id[4:]
+    if session_id and session_id.startswith('ses-'):
+        session_id = session_id[4:]
 
-    deriv_queries = {
-        'anat_mask': {
-            'datatype': 'anat',
-            'desc': 'brain',
-            'space': 'orig',
-            'suffix': 'mask',
-        },
-        'anat_aseg': {
-            'datatype': 'anat',
-            'desc': 'aseg',
-            'space': 'orig',
-            'suffix': 'dseg',
-        },
-    }
-    if derivatives_filters is not None:
-        deriv_queries.update(derivatives_filters)
+    sessions_tsv = Path(bids_root) / f'sub-{subject_id}' / f'sub-{subject_id}_sessions.tsv'
+    if sessions_tsv.exists() and session_id is not None:
+        age = _get_age_from_tsv(sessions_tsv, level='session', key=f'ses-{session_id}')
 
-    derivatives = {}
-    for deriv, query in deriv_queries.items():
-        res = layout.get(
-            scope='derivatives',
-            subject=subject_id,
-            extension=['.nii', '.nii.gz'],
-            return_type="filename",
-            **query,
-        )
-        if not res:
-            continue
-        if len(res) > 1:  # Some queries may want multiple results
-            raise Exception(
-                f"When searching for <{deriv}>, found multiple results: {[f.path for f in res]}"
-            )
-        derivatives[deriv] = res[0]
-    return derivatives
+    participants_tsv = Path(bids_root) / 'participants.tsv'
+    if participants_tsv.exists() and age is None:
+        age = _get_age_from_tsv(participants_tsv, level='participant', key=f'sub-{subject_id}')
+
+    return age
+
+
+def _get_age_from_tsv(
+    bids_tsv: Path, level: ty.Literal['session', 'participant'], key: str
+) -> int | None:
+    import pandas as pd
+
+    df = pd.read_csv(str(bids_tsv), sep='\t')
+    age_col = None
+    # prefer explicit "age_months" over "age"
+    for c in ('age_months', 'age'):
+        if c in df.columns:
+            age_col = c
+            break
+
+    if age_col == 'age':
+        # verify age is in months
+        bids_json = bids_tsv.with_suffix('.json')
+        if not _verify_age_json(bids_json):
+            warnings.warn(f'Could not verify age column is in months for file: {bids_tsv}')
+
+    # find the relevant row
+    if level == 'session':
+        mask = df.session_id == key
+    elif level == 'participant':
+        mask = df.participant_id == key
+
+    try:
+        # extract age value from row
+        age = int(df.loc[mask, age_col].values[0])
+    except Exception:
+        age = None
+    return age
+
+
+def _verify_age_json(bids_json: Path) -> bool:
+    try:
+        data = json.loads(bids_json.read_text())
+        return data['age']['Units'].lower() == 'months'
+    except Exception:
+        return False
