@@ -1,12 +1,20 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
 """Writing outputs."""
+from __future__ import annotations
+
+import typing as ty
+from pathlib import Path
+
 from nipype.interfaces import utility as niu
 from nipype.pipeline import engine as pe
 from niworkflows.engine.workflows import LiterateWorkflow as Workflow
 
 from ...interfaces import DerivativesDataSink
 
+if ty.TYPE_CHECKING:
+    from niworkflows.utils.spaces import SpatialReferences
+
 BIDS_TISSUE_ORDER = ("GM", "WM", "CSF")
 
 
@@ -74,13 +82,19 @@ def init_coreg_report_wf(*, output_dir, name="coreg_report_wf"):
     return workflow
 
 
-def init_anat_reports_wf(*, freesurfer, output_dir, sloppy, name="anat_reports_wf"):
+def init_anat_reports_wf(
+    *,
+    surface_recon: ty.Literal['freesurfer', 'infantfs', 'mcribs'] | None,
+    output_dir: str,
+    sloppy: bool,
+    name="anat_reports_wf",
+) -> Workflow:
     """
     Patched workflow for reports to allow no resolution for templates
     Set up a battery of datasinks to store reports in the right location.
     Parameters
     ----------
-    freesurfer : :obj:`bool`
+    recon_method : :obj:`bool`
         FreeSurfer was enabled
     output_dir : :obj:`str`
         Directory in which to save derivatives
@@ -123,62 +137,63 @@ def init_anat_reports_wf(*, freesurfer, output_dir, sloppy, name="anat_reports_w
         _rpt_masks,
     )
 
-    from ...utils.patches import set_tf_resolution
-
     workflow = Workflow(name=name)
 
     inputfields = [
         "source_file",
-        "t1w_conform_report",
-        "t1w_preproc",
-        "t1w_dseg",
-        "t1w_mask",
+        "anat_conform_report",
+        "anat_preproc",
+        "anat_dseg",
+        "anat_mask",
         "template",
         "std_t1w",
         "std_mask",
         "subject_id",
         "subjects_dir",
+        "surfaces",
+        "morphometrics",
     ]
     inputnode = pe.Node(niu.IdentityInterface(fields=inputfields), name="inputnode")
 
     seg_rpt = pe.Node(ROIsPlot(colors=["b", "magenta"], levels=[1.5, 2.5]), name="seg_rpt")
 
-    t1w_conform_check = pe.Node(
+    anat_conform_check = pe.Node(
         niu.Function(function=_empty_report),
-        name="t1w_conform_check",
+        name="anat_conform_check",
         run_without_submitting=True,
     )
 
-    ds_t1w_conform_report = pe.Node(
+    ds_anat_conform_report = pe.Node(
         DerivativesDataSink(base_directory=output_dir, desc="conform", datatype="figures"),
-        name="ds_t1w_conform_report",
+        name="ds_anat_conform_report",
         run_without_submitting=True,
     )
 
-    ds_t1w_dseg_mask_report = pe.Node(
+    ds_anat_dseg_mask_report = pe.Node(
         DerivativesDataSink(base_directory=output_dir, suffix="dseg", datatype="figures"),
-        name="ds_t1w_dseg_mask_report",
+        name="ds_anat_dseg_mask_report",
         run_without_submitting=True,
     )
 
     # fmt: off
     workflow.connect([
-        (inputnode, t1w_conform_check, [('t1w_conform_report', 'in_file')]),
-        (t1w_conform_check, ds_t1w_conform_report, [('out', 'in_file')]),
-        (inputnode, ds_t1w_conform_report, [('source_file', 'source_file')]),
-        (inputnode, ds_t1w_dseg_mask_report, [('source_file', 'source_file')]),
-        (inputnode, seg_rpt, [('t1w_preproc', 'in_file'),
-                              ('t1w_mask', 'in_mask'),
-                              ('t1w_dseg', 'in_rois')]),
-        (seg_rpt, ds_t1w_dseg_mask_report, [('out_report', 'in_file')]),
+        (inputnode, anat_conform_check, [('anat_conform_report', 'in_file')]),
+        (anat_conform_check, ds_anat_conform_report, [('out', 'in_file')]),
+        (inputnode, ds_anat_conform_report, [('source_file', 'source_file')]),
+        (inputnode, ds_anat_dseg_mask_report, [('source_file', 'source_file')]),
+        (inputnode, seg_rpt, [('anat_preproc', 'in_file'),
+                              ('anat_mask', 'in_mask'),
+                              ('anat_dseg', 'in_rois')]),
+        (seg_rpt, ds_anat_dseg_mask_report, [('out_report', 'in_file')]),
     ])
     # fmt: on
 
     # Generate reportlets showing spatial normalization
-    tf_select = pe.Node(TemplateFlowSelect(), name="tf_select", run_without_submitting=True)
-
-    set_tf_res = pe.Node(niu.Function(function=set_tf_resolution), name="set_tf_res")
-    set_tf_res.inputs.sloppy = sloppy
+    tf_select = pe.Node(
+        TemplateFlowSelect(resolution=1),
+        name="tf_select",
+        run_without_submitting=True,
+    )
 
     norm_msk = pe.Node(
         niu.Function(
@@ -199,8 +214,6 @@ def init_anat_reports_wf(*, freesurfer, output_dir, sloppy, name="anat_reports_w
 
     # fmt: off
     workflow.connect([
-        (inputnode, set_tf_res, [(('template', _drop_cohort), 'template')]),
-        (set_tf_res, tf_select, [('out', 'resolution')]),
         (inputnode, tf_select, [(('template', _drop_cohort), 'template'),
                                 (('template', _pick_cohort), 'cohort')]),
         (inputnode, norm_rpt, [('template', 'before_label')]),
@@ -217,38 +230,43 @@ def init_anat_reports_wf(*, freesurfer, output_dir, sloppy, name="anat_reports_w
     ])
     # fmt: on
 
-    if freesurfer:
-        from smriprep.interfaces.reports import FSSurfaceReport
+    if not surface_recon:
+        return workflow
 
-        recon_report = pe.Node(FSSurfaceReport(), name="recon_report")
-        recon_report.interface._always_run = True
+    # TODO: Separate report for MCRIBS?
+    from smriprep.interfaces.reports import FSSurfaceReport
 
-        ds_recon_report = pe.Node(
-            DerivativesDataSink(base_directory=output_dir, desc="reconall", datatype="figures"),
-            name="ds_recon_report",
-            run_without_submitting=True,
-        )
-        # fmt: off
-        workflow.connect([
-            (inputnode, recon_report, [('subjects_dir', 'subjects_dir'),
-                                       ('subject_id', 'subject_id')]),
-            (recon_report, ds_recon_report, [('out_report', 'in_file')]),
-            (inputnode, ds_recon_report, [('source_file', 'source_file')])
-        ])
-        # fmt: on
+    recon_report = pe.Node(FSSurfaceReport(), name="recon_report")
+    recon_report.interface._always_run = True
+
+    ds_recon_report = pe.Node(
+        DerivativesDataSink(base_directory=output_dir, desc="reconall", datatype="figures"),
+        name="ds_recon_report",
+        run_without_submitting=True,
+    )
+    # fmt: off
+    workflow.connect([
+        (inputnode, recon_report, [('subjects_dir', 'subjects_dir'),
+                                   ('subject_id', 'subject_id')]),
+        (recon_report, ds_recon_report, [('out_report', 'in_file')]),
+        (inputnode, ds_recon_report, [('source_file', 'source_file')])
+    ])
+    # fmt: on
 
     return workflow
 
 
 def init_anat_derivatives_wf(
     *,
-    bids_root,
-    freesurfer,
-    num_t1w,
-    output_dir,
-    spaces,
-    name="anat_derivatives_wf",
-    tpm_labels=BIDS_TISSUE_ORDER,
+    bids_root: Path | str,
+    output_dir: Path | str,
+    spaces: SpatialReferences,
+    cifti_output: bool,
+    num_t1w: int | None,
+    num_t2w: int | None,
+    surface_recon: ty.Literal['freesurfer', 'infantfs', 'mcribs'] | None,
+    tpm_labels: ty.Tuple[str, str, str] = BIDS_TISSUE_ORDER,
+    name: str = "anat_derivatives_wf",
 ):
     """
     Set up a battery of datasinks to store derivatives in the right location.
@@ -308,6 +326,16 @@ def init_anat_derivatives_wf(
         FreeSurfer's aseg segmentation, in native T1w space
     t1w_fs_aparc
         FreeSurfer's aparc+aseg segmentation, in native T1w space
+    t2w_source_files
+        List of input T2w images
+    t2w_preproc
+        The T2w image in T1w space.
+    cifti_morph
+        Morphometric CIFTI-2 dscalar files
+    cifti_density
+        Grayordinate density
+    cifti_metadata
+        JSON files containing metadata dictionaries
     """
     from niworkflows.interfaces.nibabel import ApplyMask
     from niworkflows.interfaces.utility import KeySelect
@@ -325,122 +353,203 @@ def init_anat_derivatives_wf(
         niu.IdentityInterface(
             fields=[
                 "template",
-                "source_files",
+                # T1w
+                "t1w_source_files",
                 "t1w_ref_xfms",
                 "t1w_preproc",
-                "t1w_mask",
-                "t1w_dseg",
-                "t1w_tpms",
+                # T2w
+                "t2w_source_files",
+                "t2w_ref_xfms",
+                "t2w_preproc",
+                # Can be in either T1w/T2w space
+                "anat_mask",
+                "anat_dseg",
+                "anat_tpms",
                 "anat2std_xfm",
                 "std2anat_xfm",
-                "t1w2fsnative_xfm",
-                "fsnative2t1w_xfm",
+                "anat2fsnative_xfm",
+                "fsnative2anat_xfm",
+                # FS
+                "anat_fs_aseg",
+                "anat_fs_aparc",
+                "anat_ribbon",
                 "surfaces",
-                "t1w_fs_aseg",
-                "t1w_fs_aparc",
+                "morphometrics",
+                # CIFTI
+                "cifti_metadata",
+                "cifti_density",
+                "cifti_morph",
+                "sphere_reg",
+                "sphere_reg_fsLR",
             ]
         ),
         name="inputnode",
     )
+    # The preferred space to use for to/from entities
+    source_files = "t1w_source_files" if num_t1w else "t2w_source_files"
+    space = "T1w" if num_t1w else "T2w"
 
-    raw_sources = pe.Node(niu.Function(function=_bids_relative), name="raw_sources")
-    raw_sources.inputs.bids_root = bids_root
+    if num_t1w:
+        raw_sources = pe.Node(niu.Function(function=_bids_relative), name="t1w_raw_sources")
+        raw_sources.inputs.bids_root = bids_root
 
-    ds_t1w_preproc = pe.Node(
-        DerivativesDataSink(base_directory=output_dir, desc="preproc", compress=True),
-        name="ds_t1w_preproc",
-        run_without_submitting=True,
-    )
-    ds_t1w_preproc.inputs.SkullStripped = False
+        ds_t1w_preproc = pe.Node(
+            DerivativesDataSink(base_directory=output_dir, desc="preproc", compress=True),
+            name="ds_t1w_preproc",
+            run_without_submitting=True,
+        )
+        ds_t1w_preproc.inputs.SkullStripped = False
 
-    ds_t1w_mask = pe.Node(
+        if num_t1w > 1:
+            # Please note the dictionary unpacking to provide the from argument.
+            # It is necessary because from is a protected keyword (not allowed as argument name).
+            ds_t1w_ref_xfms = pe.MapNode(
+                DerivativesDataSink(
+                    base_directory=output_dir,
+                    to="T1w",
+                    mode="image",
+                    suffix="xfm",
+                    extension="txt",
+                    **{"from": "orig"},
+                ),
+                iterfield=["source_file", "in_file"],
+                name="ds_t1w_ref_xfms",
+                run_without_submitting=True,
+            )
+            # fmt:off
+            workflow.connect([
+                (inputnode, ds_t1w_ref_xfms, [('t1w_source_files', 'source_file'),
+                                              ('t1w_ref_xfms', 'in_file')]),
+            ])
+            # fmt:on
+
+    if num_t2w:
+        if not num_t1w:
+            raw_sources = pe.Node(niu.Function(function=_bids_relative), name="t2w_raw_sources")
+            raw_sources.inputs.bids_root = bids_root
+
+        ds_t2w_preproc = pe.Node(
+            DerivativesDataSink(base_directory=output_dir, desc="preproc", compress=True),
+            name="ds_t2w_preproc",
+            run_without_submitting=True,
+        )
+        ds_t2w_preproc.inputs.SkullStripped = False
+
+        if num_t1w:
+            ds_t2w_preproc.inputs.space = "T1w"
+
+        if num_t2w > 1:
+            # Please note the dictionary unpacking to provide the from argument.
+            # It is necessary because from is a protected keyword (not allowed as argument name).
+            ds_t2w_ref_xfms = pe.MapNode(
+                DerivativesDataSink(
+                    base_directory=output_dir,
+                    to="T1w",
+                    mode="image",
+                    suffix="xfm",
+                    extension="txt",
+                    **{"from": "orig"},
+                ),
+                iterfield=["source_file", "in_file"],
+                name="ds_t2w_ref_xfms",
+                run_without_submitting=True,
+            )
+            # fmt:off
+            workflow.connect([
+                (inputnode, ds_t2w_ref_xfms, [('t2w_source_files', 'source_file'),
+                                              ('t2w_ref_xfms', 'in_file')]),
+            ])
+            # fmt:on
+
+    ds_anat_mask = pe.Node(
         DerivativesDataSink(base_directory=output_dir, desc="brain", suffix="mask", compress=True),
-        name="ds_t1w_mask",
+        name="ds_anat_mask",
         run_without_submitting=True,
     )
-    ds_t1w_mask.inputs.Type = "Brain"
+    ds_anat_mask.inputs.Type = "Brain"
 
-    ds_t1w_dseg = pe.Node(
+    ds_anat_dseg = pe.Node(
         DerivativesDataSink(base_directory=output_dir, suffix="dseg", compress=True),
-        name="ds_t1w_dseg",
+        name="ds_anat_dseg",
         run_without_submitting=True,
     )
 
-    ds_t1w_tpms = pe.Node(
+    ds_anat_tpms = pe.Node(
         DerivativesDataSink(base_directory=output_dir, suffix="probseg", compress=True),
-        name="ds_t1w_tpms",
+        name="ds_anat_tpms",
         run_without_submitting=True,
     )
-    ds_t1w_tpms.inputs.label = tpm_labels
+    ds_anat_tpms.inputs.label = tpm_labels
 
-    # fmt: off
+    ds_anat_ribbon = pe.Node(
+        DerivativesDataSink(
+            base_directory=output_dir,
+            desc="ribbon",
+            suffix="mask",
+            extension=".nii.gz",
+            compress=True,
+        ),
+        name="ds_anat_ribbon",
+        run_without_submitting=True,
+    )
+
+    if num_t1w:
+        workflow.connect([
+            (inputnode, ds_t1w_preproc, [('t1w_preproc', 'in_file'),
+                                         ('t1w_source_files', 'source_file')]),
+        ])  # fmt:skip
+
+    if num_t2w:
+        workflow.connect([
+            (inputnode, ds_t2w_preproc, [('t2w_preproc', 'in_file'),
+                                         ('t2w_source_files', 'source_file')]),
+        ])  # fmt:skip
+
+    # fmt:off
     workflow.connect([
-        (inputnode, raw_sources, [('source_files', 'in_files')]),
-        (inputnode, ds_t1w_preproc, [('t1w_preproc', 'in_file'),
-                                     ('source_files', 'source_file')]),
-        (inputnode, ds_t1w_mask, [('t1w_mask', 'in_file'),
-                                  ('source_files', 'source_file')]),
-        (inputnode, ds_t1w_tpms, [('t1w_tpms', 'in_file'),
-                                  ('source_files', 'source_file')]),
-        (inputnode, ds_t1w_dseg, [('t1w_dseg', 'in_file'),
-                                  ('source_files', 'source_file')]),
-        (raw_sources, ds_t1w_mask, [('out', 'RawSources')]),
+        (inputnode, raw_sources, [(source_files, 'in_files')]),
+        (inputnode, ds_anat_mask, [('anat_mask', 'in_file'),
+                                   (source_files, 'source_file')]),
+        (inputnode, ds_anat_tpms, [('anat_tpms', 'in_file'),
+                                   (source_files, 'source_file')]),
+        (inputnode, ds_anat_dseg, [('anat_dseg', 'in_file'),
+                                   (source_files, 'source_file')]),
+        (inputnode, ds_anat_ribbon, [('anat_ribbon', 'in_file'),
+                                     (source_files, 'source_file')]),
+        (raw_sources, ds_anat_mask, [('out', 'RawSources')]),
     ])
-    # fmt: on
+    # fmt:on
 
     # Transforms
     if spaces.get_spaces(nonstandard=False, dim=(3,)):
-        ds_std2t1w_xfm = pe.MapNode(
-            DerivativesDataSink(base_directory=output_dir, to="T1w", mode="image", suffix="xfm"),
+        ds_std2anat_xfm = pe.MapNode(
+            DerivativesDataSink(base_directory=output_dir, to=space, mode="image", suffix="xfm"),
             iterfield=("in_file", "from"),
-            name="ds_std2t1w_xfm",
+            name="ds_std2anat_xfm",
             run_without_submitting=True,
         )
 
-        ds_t1w2std_xfm = pe.MapNode(
+        ds_anat2std_xfm = pe.MapNode(
             DerivativesDataSink(
-                base_directory=output_dir, mode="image", suffix="xfm", **{"from": "T1w"}
+                base_directory=output_dir, mode="image", suffix="xfm", **{"from": space}
             ),
             iterfield=("in_file", "to"),
-            name="ds_t1w2std_xfm",
+            name="ds_anat2std_xfm",
             run_without_submitting=True,
         )
 
-        # fmt: off
+        # fmt:off
         workflow.connect([
-            (inputnode, ds_t1w2std_xfm, [
+            (inputnode, ds_anat2std_xfm, [
                 ('anat2std_xfm', 'in_file'),
                 (('template', _combine_cohort), 'to'),
-                ('source_files', 'source_file')]),
-            (inputnode, ds_std2t1w_xfm, [
+                (source_files, 'source_file')]),
+            (inputnode, ds_std2anat_xfm, [
                 ('std2anat_xfm', 'in_file'),
                 (('template', _combine_cohort), 'from'),
-                ('source_files', 'source_file')]),
+                (source_files, 'source_file')]),
         ])
-        # fmt: on
-
-    if num_t1w > 1:
-        # Please note the dictionary unpacking to provide the from argument.
-        # It is necessary because from is a protected keyword (not allowed as argument name).
-        ds_t1w_ref_xfms = pe.MapNode(
-            DerivativesDataSink(
-                base_directory=output_dir,
-                to="T1w",
-                mode="image",
-                suffix="xfm",
-                extension="txt",
-                **{"from": "orig"},
-            ),
-            iterfield=["source_file", "in_file"],
-            name="ds_t1w_ref_xfms",
-            run_without_submitting=True,
-        )
-        # fmt: off
-        workflow.connect([
-            (inputnode, ds_t1w_ref_xfms, [('source_files', 'source_file'),
-                                          ('t1w_ref_xfms', 'in_file')]),
-        ])
-        # fmt: on
+        # fmt:on
 
     # Write derivatives in standard spaces specified by --output-spaces
     if getattr(spaces, "_cached") is not None and spaces.cached.references:
@@ -472,8 +581,7 @@ def init_anat_derivatives_wf(
 
         gen_ref = pe.Node(GenerateSamplingReference(), name="gen_ref", mem_gb=0.01)
 
-        # Mask T1w preproc images
-        mask_t1w = pe.Node(ApplyMask(), name="mask_t1w")
+        mask_anat = pe.Node(ApplyMask(), name="mask_anat")
 
         # Resample T1w-space inputs
         anat2std_t1w = pe.Node(
@@ -538,15 +646,17 @@ def init_anat_derivatives_wf(
         #           output in the data/io_spec.json file.
         ds_std_tpms.inputs.label = tpm_labels
 
+        preproc_file = 't1w_preproc' if num_t1w else 't2w_preproc'
+
         # fmt: off
         workflow.connect([
-            (inputnode, mask_t1w, [('t1w_preproc', 'in_file'),
-                                   ('t1w_mask', 'in_mask')]),
-            (mask_t1w, anat2std_t1w, [('out_file', 'input_image')]),
-            (inputnode, anat2std_mask, [('t1w_mask', 'input_image')]),
-            (inputnode, anat2std_dseg, [('t1w_dseg', 'input_image')]),
-            (inputnode, anat2std_tpms, [('t1w_tpms', 'input_image')]),
-            (inputnode, gen_ref, [('t1w_preproc', 'moving_image')]),
+            (inputnode, mask_anat, [(preproc_file, 'in_file'),
+                                    ('anat_mask', 'in_mask')]),
+            (mask_anat, anat2std_t1w, [('out_file', 'input_image')]),
+            (inputnode, anat2std_mask, [('anat_mask', 'input_image')]),
+            (inputnode, anat2std_dseg, [('anat_dseg', 'input_image')]),
+            (inputnode, anat2std_tpms, [('anat_tpms', 'input_image')]),
+            (inputnode, gen_ref, [(preproc_file, 'moving_image')]),
             (inputnode, select_xfm, [
                 ('anat2std_xfm', 'anat2std_xfm'),
                 ('template', 'keys')]),
@@ -579,7 +689,7 @@ def init_anat_derivatives_wf(
             ]
             # Connect the source_file input of these datasinks
             + [
-                (inputnode, n, [("source_files", "source_file")])
+                (inputnode, n, [(source_files, "source_file")])
                 for n in (ds_std_t1w, ds_std_mask, ds_std_dseg, ds_std_tpms)
             ]
             # Connect the space input of these datasinks
@@ -593,7 +703,7 @@ def init_anat_derivatives_wf(
             ]
         )
 
-    if not freesurfer:
+    if not surface_recon:
         return workflow
 
     from niworkflows.interfaces.nitransforms import ConcatenateXFMs
@@ -602,28 +712,28 @@ def init_anat_derivatives_wf(
     # FS native space transforms
     lta2itk_fwd = pe.Node(ConcatenateXFMs(), name="lta2itk_fwd", run_without_submitting=True)
     lta2itk_inv = pe.Node(ConcatenateXFMs(), name="lta2itk_inv", run_without_submitting=True)
-    ds_t1w_fsnative = pe.Node(
+    ds_anat_fsnative = pe.Node(
         DerivativesDataSink(
             base_directory=output_dir,
             mode="image",
             to="fsnative",
             suffix="xfm",
             extension="txt",
-            **{"from": "T1w"},
+            **{"from": space},
         ),
-        name="ds_t1w_fsnative",
+        name="ds_anat_fsnative",
         run_without_submitting=True,
     )
-    ds_fsnative_t1w = pe.Node(
+    ds_fsnative_anat = pe.Node(
         DerivativesDataSink(
             base_directory=output_dir,
             mode="image",
-            to="T1w",
+            to=space,
             suffix="xfm",
             extension="txt",
             **{"from": "fsnative"},
         ),
-        name="ds_fsnative_t1w",
+        name="ds_fsnative_anat",
         run_without_submitting=True,
     )
     # Surfaces
@@ -636,39 +746,114 @@ def init_anat_derivatives_wf(
         name="ds_surfs",
         run_without_submitting=True,
     )
-    # Parcellations
-    ds_t1w_fsaseg = pe.Node(
-        DerivativesDataSink(base_directory=output_dir, desc="aseg", suffix="dseg", compress=True),
-        name="ds_t1w_fsaseg",
+    name_regs = pe.MapNode(
+        Path2BIDS(), iterfield="in_file", name="name_regs", run_without_submitting=True
+    )
+    ds_regs = pe.MapNode(
+        DerivativesDataSink(
+            base_directory=output_dir,
+            desc="reg",
+            suffix="sphere",
+            extension=".surf.gii",
+        ),
+        iterfield=["in_file", "hemi"],
+        name="ds_regs",
         run_without_submitting=True,
     )
-    ds_t1w_fsparc = pe.Node(
+    name_reg_fsLR = pe.MapNode(
+        Path2BIDS(), iterfield="in_file", name="name_reg_fsLR", run_without_submitting=True
+    )
+    ds_reg_fsLR = pe.MapNode(
+        DerivativesDataSink(
+            base_directory=output_dir,
+            space="fsLR",
+            desc="reg",
+            suffix="sphere",
+            extension=".surf.gii",
+        ),
+        iterfield=["in_file", "hemi"],
+        name="ds_reg_fsLR",
+        run_without_submitting=True,
+    )
+    # Morphometrics
+    name_morphs = pe.MapNode(
+        Path2BIDS(),
+        iterfield="in_file",
+        name="name_morphs",
+        run_without_submitting=True,
+    )
+    ds_morphs = pe.MapNode(
+        DerivativesDataSink(base_directory=output_dir, extension=".shape.gii"),
+        iterfield=["in_file", "hemi", "suffix"],
+        name="ds_morphs",
+        run_without_submitting=True,
+    )
+    # Parcellations
+    ds_anat_fsaseg = pe.Node(
+        DerivativesDataSink(base_directory=output_dir, desc="aseg", suffix="dseg", compress=True),
+        name="ds_anat_fsaseg",
+        run_without_submitting=True,
+    )
+    ds_anat_fsparc = pe.Node(
         DerivativesDataSink(
             base_directory=output_dir, desc="aparcaseg", suffix="dseg", compress=True
         ),
-        name="ds_t1w_fsparc",
+        name="ds_anat_fsparc",
         run_without_submitting=True,
     )
 
     # fmt: off
     workflow.connect([
-        (inputnode, lta2itk_fwd, [('t1w2fsnative_xfm', 'in_xfms')]),
-        (inputnode, lta2itk_inv, [('fsnative2t1w_xfm', 'in_xfms')]),
-        (inputnode, ds_t1w_fsnative, [('source_files', 'source_file')]),
-        (lta2itk_fwd, ds_t1w_fsnative, [('out_xfm', 'in_file')]),
-        (inputnode, ds_fsnative_t1w, [('source_files', 'source_file')]),
-        (lta2itk_inv, ds_fsnative_t1w, [('out_xfm', 'in_file')]),
+        (inputnode, lta2itk_fwd, [('anat2fsnative_xfm', 'in_xfms')]),
+        (inputnode, lta2itk_inv, [('fsnative2anat_xfm', 'in_xfms')]),
+        (inputnode, ds_anat_fsnative, [(source_files, 'source_file')]),
+        (lta2itk_fwd, ds_anat_fsnative, [('out_xfm', 'in_file')]),
+        (inputnode, ds_fsnative_anat, [(source_files, 'source_file')]),
+        (lta2itk_inv, ds_fsnative_anat, [('out_xfm', 'in_file')]),
         (inputnode, name_surfs, [('surfaces', 'in_file')]),
         (inputnode, ds_surfs, [('surfaces', 'in_file'),
-                               ('source_files', 'source_file')]),
+                               (source_files, 'source_file')]),
         (name_surfs, ds_surfs, [('hemi', 'hemi'),
                                 ('suffix', 'suffix')]),
-        (inputnode, ds_t1w_fsaseg, [('t1w_fs_aseg', 'in_file'),
-                                    ('source_files', 'source_file')]),
-        (inputnode, ds_t1w_fsparc, [('t1w_fs_aparc', 'in_file'),
-                                    ('source_files', 'source_file')]),
+        (inputnode, name_regs, [('sphere_reg', 'in_file')]),
+        (inputnode, ds_regs, [('sphere_reg', 'in_file'),
+                              (source_files, 'source_file')]),
+        (name_regs, ds_regs, [('hemi', 'hemi')]),
+        (inputnode, name_reg_fsLR, [('sphere_reg_fsLR', 'in_file')]),
+        (inputnode, ds_reg_fsLR, [('sphere_reg_fsLR', 'in_file'),
+                                  (source_files, 'source_file')]),
+        (name_reg_fsLR, ds_reg_fsLR, [('hemi', 'hemi')]),
+        (inputnode, name_morphs, [('morphometrics', 'in_file')]),
+        (inputnode, ds_morphs, [('morphometrics', 'in_file'),
+                                (source_files, 'source_file')]),
+        (name_morphs, ds_morphs, [('hemi', 'hemi'),
+                                  ('suffix', 'suffix')]),
+        (inputnode, ds_anat_fsaseg, [('anat_fs_aseg', 'in_file'),
+                                     (source_files, 'source_file')]),
+        (inputnode, ds_anat_fsparc, [('anat_fs_aparc', 'in_file'),
+                                     (source_files, 'source_file')]),
     ])
     # fmt: on
+    if cifti_output:
+        ds_cifti_morph = pe.MapNode(
+            DerivativesDataSink(
+                base_directory=output_dir,
+                suffix=['curv', 'sulc', 'thickness'],
+                compress=False,
+                space='fsLR',
+            ),
+            name='ds_cifti_morph',
+            run_without_submitting=True,
+            iterfield=["in_file", "meta_dict", "suffix"],
+        )
+        # fmt:off
+        workflow.connect([
+            (inputnode, ds_cifti_morph, [('cifti_morph', 'in_file'),
+                                         (source_files, 'source_file'),
+                                         ('cifti_density', 'density'),
+                                         (('cifti_metadata', _read_jsons), 'meta_dict')])
+        ])
+        # fmt:on
     return workflow
 
 
@@ -681,3 +866,10 @@ def _set_tpl_res(space, resolution):
         return int(resolution)
     except ValueError:
         return 1
+
+
+def _read_jsons(in_file):
+    from json import loads
+    from pathlib import Path
+
+    return [loads(Path(f).read_text()) for f in in_file]
