@@ -6,6 +6,7 @@ from nipype.interfaces.ants.segmentation import JointFusion
 from nipype.pipeline import engine as pe
 from niworkflows.interfaces.fixes import FixHeaderApplyTransforms as ApplyTransforms
 from niworkflows.interfaces.fixes import FixHeaderRegistration as Registration
+from niworkflows.interfaces.nibabel import MapLabels
 from pkg_resources import resource_filename as pkgr_fn
 from smriprep.utils.misc import apply_lut as _apply_bids_lut
 from smriprep.workflows.anatomical import (
@@ -36,11 +37,6 @@ def init_anat_segmentations_wf(
     from niworkflows.engine.workflows import LiterateWorkflow as Workflow
     from niworkflows.utils.connections import listify
 
-    if anat_modality != "T1w":
-        raise NotImplementedError(
-            "Only T1w images are currently accepted for the segmentation workflow."
-        )
-
     if precomp_aseg and template_dir:
         print("Found precomputed aseg; skipping JointLabelFusion", file=sys.stderr)
         template_dir = None
@@ -54,6 +50,7 @@ def init_anat_segmentations_wf(
         niu.IdentityInterface(fields=["anat_aseg", "anat_dseg", "anat_tpms"]),
         name="outputnode",
     )
+    buffernode = pe.Node(niu.IdentityInterface(fields=["final_aseg"]), name='buffernode')
 
     wf.__desc__ = """Brain tissue segmentation of cerebrospinal fluid (CSF),
 white-matter (WM) and gray-matter (GM) was performed on """
@@ -65,7 +62,8 @@ white-matter (WM) and gray-matter (GM) was performed on """
         from nipype.interfaces.fsl.base import Info as FSLInfo
 
         wf.__desc__ += (
-            f"the brain-extracted T1w using FSL FAST {FSLInfo.version() or '(version unknown)'}."
+            f"the brain-extracted {anat_modality} using FSL FAST "
+            f"{FSLInfo.version() or '(version unknown)'}."
         )
 
         # Use FSL FAST for segmentations
@@ -97,7 +95,7 @@ white-matter (WM) and gray-matter (GM) was performed on """
         from nipype.interfaces.ants.base import Info as ANTsInfo
 
         wf.__desc__ += (
-            "the brain-extracted T1w using ANTs JointFusion, distributed with ANTs "
+            f"the brain-extracted {anat_modality} using ANTs JointFusion, distributed with ANTs "
             f"{ANTsInfo.version() or '(version unknown)'}."
         )
         tmpl_anats, tmpl_segs = _parse_segmentation_atlases(anat_modality, template_dir)
@@ -159,31 +157,30 @@ white-matter (WM) and gray-matter (GM) was performed on """
             (apply_atlas, jointfusion, [('output_image', 'atlas_image')]),
             (apply_seg, jointfusion, [('output_image', 'atlas_segmentation_image')]),
             (jointfusion, jf_label, [('out_label_fusion', 'in_file')]),
+            (jf_label, buffernode, [('out_file', 'final_aseg')]),
         ])
         # fmt:on
 
     elif precomp_aseg:
         wf.__desc__ += "a pre-computed segmentation."
-        from niworkflows.interfaces.header import ValidateImage
 
-        inputnode.inputs.anat_aseg = precomp_aseg
-        validate_aseg = pe.Node(ValidateImage(), name="validate_aseg")
-        wf.connect(inputnode, "anat_aseg", validate_aseg, "in_file")
+        wf.connect(inputnode, "anat_aseg", buffernode, "final_aseg")
 
     # Otherwise, the final aseg will be split into three tissue classes
     # regardless if it was precomputed or generated via JLF
     lut_anat_dseg.inputs.lut = _aseg_to_three()
     split_seg = pe.Node(niu.Function(function=_split_segments), name="split_seg")
-    out_aseg = validate_aseg if precomp_aseg else jf_label
+
     # fmt: off
     wf.connect([
-        (out_aseg, outputnode, [('out_file', 'anat_aseg')]),
-        (out_aseg, lut_anat_dseg, [('out_file', 'in_dseg')]),
+        (buffernode, outputnode, [('final_aseg', 'anat_aseg')]),
+        (buffernode, lut_anat_dseg, [('final_aseg', 'in_dseg')]),
         (lut_anat_dseg, outputnode, [('out', 'anat_dseg')]),
         (lut_anat_dseg, split_seg, [('out', 'in_file')]),
         (split_seg, outputnode, [('out', 'anat_tpms')]),
     ])
     # fmt: on
+
     return wf
 
 
