@@ -12,7 +12,7 @@
 #     This change is to treat sessions as a "first-class" identifier, to better handle the
 #     potential rapid changing of brain morphometry.
 #
-# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+# Copyright 2023 The NiPreps Developers <nipreps@gmail.com>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -39,23 +39,28 @@ NiBabies base processing workflows
 .. autofunction:: init_single_subject_wf
 
 """
+from __future__ import annotations
 
 import os
 import sys
+import typing as ty
 from copy import deepcopy
 
 from nipype.interfaces import utility as niu
 from nipype.pipeline import engine as pe
 from packaging.version import Version
 
-from .. import config
-from ..interfaces import DerivativesDataSink
-from ..interfaces.reports import AboutSummary, SubjectSummary
-from ..utils.bids import group_bolds_ref
-from .bold import init_func_preproc_wf
+from nibabies import config
+from nibabies.interfaces import DerivativesDataSink
+from nibabies.interfaces.reports import AboutSummary, SubjectSummary
+from nibabies.utils.bids import parse_bids_for_age_months
+from nibabies.workflows.bold import init_func_preproc_wf
+
+if ty.TYPE_CHECKING:
+    from niworkflows.utils.spaces import SpatialReferences
 
 
-def init_nibabies_wf(participants_table):
+def init_nibabies_wf(subworkflows_list):
     """
     Build *NiBabies*'s pipeline.
 
@@ -77,8 +82,9 @@ def init_nibabies_wf(participants_table):
 
     Parameters
     ----------
-    participants_table: :obj:`dict`
-        Keys of participant labels and values of the sessions to process.
+    subworkflows_list: :obj:`list` of :obj:`tuple`
+        A list of the subworkflows to create.
+        Each subject session is run as an individual workflow.
     """
     from niworkflows.engine.workflows import LiterateWorkflow as Workflow
     from niworkflows.interfaces.bids import BIDSFreeSurferDir
@@ -87,13 +93,15 @@ def init_nibabies_wf(participants_table):
     nibabies_wf = Workflow(name=f"nibabies_{ver.major}_{ver.minor}_wf")
     nibabies_wf.base_dir = config.execution.work_dir
 
+    execution_spaces = init_execution_spaces()
+
     freesurfer = config.workflow.run_reconall
     if freesurfer:
         fsdir = pe.Node(
             BIDSFreeSurferDir(
                 derivatives=config.execution.output_dir,
                 freesurfer_home=os.getenv("FREESURFER_HOME"),
-                spaces=config.workflow.spaces.get_fs_spaces(),
+                spaces=execution_spaces.get_fs_spaces(),
             ),
             name=f"fsdir_run_{config.execution.run_uuid.replace('-', '_')}",
             run_without_submitting=True,
@@ -101,38 +109,60 @@ def init_nibabies_wf(participants_table):
         if config.execution.fs_subjects_dir is not None:
             fsdir.inputs.subjects_dir = str(config.execution.fs_subjects_dir.absolute())
 
-    for subject_id, sessions in participants_table.items():
-        for session_id in sessions:
-            single_subject_wf = init_single_subject_wf(subject_id, session_id=session_id)
-
-            bids_level = [f"sub-{subject_id}"]
-            if session_id:
-                bids_level.append(f"ses-{session_id}")
-
-            log_dir = (
-                config.execution.nibabies_dir.joinpath(*bids_level)
-                / "log"
-                / config.execution.run_uuid
+    for subject_id, session_id in subworkflows_list:
+        # Calculate the age and age-specific spaces
+        age = parse_bids_for_age_months(config.execution.bids_dir, subject_id, session_id)
+        if config.workflow.age_months:
+            config.loggers.cli.warning(
+                "`--age-months` is deprecated and will be removed in a future release."
+                "Please use a `sessions.tsv` or `participants.tsv` file to track participants age."
             )
-
-            single_subject_wf.config["execution"]["crashdump_dir"] = str(log_dir)
-            for node in single_subject_wf._get_all_nodes():
-                node.config = deepcopy(single_subject_wf.config)
-            if freesurfer:
-                nibabies_wf.connect(
-                    fsdir, "subjects_dir", single_subject_wf, "inputnode.subjects_dir"
+            age = config.workflow.age_months
+        if age is None:
+            raise RuntimeError(
+                "Could not find age for sub-{subject}{session}".format(
+                    subject=subject_id, session=f'_ses-{session_id}' if session_id else ''
                 )
-            else:
-                nibabies_wf.add_nodes([single_subject_wf])
+            )
+        output_spaces = init_workflow_spaces(execution_spaces, age)
 
-            # Dump a copy of the config file into the log directory
-            log_dir.mkdir(exist_ok=True, parents=True)
-            config.to_filename(log_dir / "nibabies.toml")
+        # skull strip template cohort
+        single_subject_wf = init_single_subject_wf(
+            subject_id,
+            session_id=session_id,
+            age=age,
+            spaces=output_spaces,
+        )
+
+        bids_level = [f"sub-{subject_id}"]
+        if session_id:
+            bids_level.append(f"ses-{session_id}")
+
+        log_dir = (
+            config.execution.nibabies_dir.joinpath(*bids_level) / "log" / config.execution.run_uuid
+        )
+
+        single_subject_wf.config["execution"]["crashdump_dir"] = str(log_dir)
+        for node in single_subject_wf._get_all_nodes():
+            node.config = deepcopy(single_subject_wf.config)
+        if freesurfer:
+            nibabies_wf.connect(fsdir, "subjects_dir", single_subject_wf, "inputnode.subjects_dir")
+        else:
+            nibabies_wf.add_nodes([single_subject_wf])
+
+        # Dump a copy of the config file into the log directory
+        log_dir.mkdir(exist_ok=True, parents=True)
+        config.to_filename(log_dir / "nibabies.toml")
 
     return nibabies_wf
 
 
-def init_single_subject_wf(subject_id, session_id=None):
+def init_single_subject_wf(
+    subject_id: str,
+    session_id: str | None = None,
+    age: int | None = None,
+    spaces: SpatialReferences | None = None,
+):
     """
     Organize the preprocessing pipeline for a single subject, at a single session.
 
@@ -159,6 +189,8 @@ def init_single_subject_wf(subject_id, session_id=None):
         Subject label for this single-subject workflow.
     session_id : :obj:`str` or None
         Session identifier.
+    age: :obj:`int` or None
+        Age (in months) of subject.
 
     Inputs
     ------
@@ -172,8 +204,9 @@ def init_single_subject_wf(subject_id, session_id=None):
     from niworkflows.utils.bids import collect_data
     from niworkflows.utils.spaces import Reference
 
+    from ..utils.bids import Derivatives
     from ..utils.misc import fix_multi_source_name
-    from .anatomical import init_infant_anat_wf
+    from .anatomical import init_infant_anat_wf, init_infant_single_anat_wf
 
     name = (
         f"single_subject_{subject_id}_{session_id}_wf"
@@ -195,9 +228,9 @@ def init_single_subject_wf(subject_id, session_id=None):
         subject_data["t2w"] = []
 
     anat_only = config.workflow.anat_only
-    derivatives = config.execution.derivatives or {}
-    anat_modality = "t1w" if subject_data["t1w"] else "t2w"
-    spaces = config.workflow.spaces
+    derivatives = Derivatives(bids_root=config.execution.layout.root)
+    contrast = "T1w" if subject_data["t1w"] else "T2w"
+    single_modality = not (subject_data['t1w'] and subject_data['t2w'])
     # Make sure we always go through these two checks
     if not anat_only and not subject_data["bold"]:
         task_id = config.execution.task_id
@@ -208,16 +241,15 @@ def init_single_subject_wf(subject_id, session_id=None):
             )
         )
 
-    if derivatives:
-        from ..utils.bids import collect_precomputed_derivatives
-
-        derivatives = collect_precomputed_derivatives(
-            config.execution.layout,
-            subject_id,
-            derivatives_filters=config.execution.derivatives_filters,
-            # session_id=None,  # TODO: Ensure session is visible at workflow level
-        )
-        config.loggers.workflow.info(f"Found precomputed derivatives: {derivatives}")
+    if config.execution.derivatives:
+        for deriv_path in config.execution.derivatives:
+            config.loggers.workflow.info("Searching for derivatives in %s", deriv_path)
+            derivatives.populate(
+                deriv_path,
+                subject_id,
+                session_id=session_id,
+            )
+        config.loggers.workflow.info("Found precomputed derivatives %s", derivatives)
 
     workflow = Workflow(name=name)
     workflow.__desc__ = """
@@ -313,15 +345,14 @@ It is released under the [CC0]\
         run_without_submitting=True,
     )
 
-    # Preprocessing of anatomical (includes registration to UNCInfant)
-    anat_preproc_wf = init_infant_anat_wf(
+    wf_args = dict(
         ants_affine_init=True,
-        age_months=config.workflow.age_months,
-        anat_modality=anat_modality,
+        age_months=age,
+        contrast=contrast,
         t1w=subject_data["t1w"],
         t2w=subject_data["t2w"],
         bids_root=config.execution.bids_dir,
-        existing_derivatives=derivatives,
+        derivatives=derivatives,
         freesurfer=config.workflow.run_reconall,
         hires=config.workflow.hires,
         longitudinal=config.workflow.longitudinal,
@@ -332,6 +363,12 @@ It is released under the [CC0]\
         skull_strip_template=Reference.from_string(config.workflow.skull_strip_template)[0],
         sloppy=config.execution.sloppy,
         spaces=spaces,
+        cifti_output=config.workflow.cifti_output,
+    )
+    anat_preproc_wf = (
+        init_infant_anat_wf(**wf_args)
+        if not single_modality
+        else init_infant_single_anat_wf(**wf_args)
     )
 
     # fmt: off
@@ -365,17 +402,17 @@ It is released under the [CC0]\
 
     workflow.connect([
         (bidssrc, bids_info, [
-            (('t1w', fix_multi_source_name), 'in_file'),
+            ((contrast.lower(), fix_multi_source_name), 'in_file'),
         ]),
         (bidssrc, summary, [
             ('t1w', 't1w'),
             ('t2w', 't2w'),
         ]),
         (bidssrc, ds_report_summary, [
-            (('t1w', fix_multi_source_name), 'source_file'),
+            ((contrast.lower(), fix_multi_source_name), 'source_file'),
         ]),
         (bidssrc, ds_report_about, [
-            (('t1w', fix_multi_source_name), 'source_file'),
+            ((contrast.lower(), fix_multi_source_name), 'source_file'),
         ]),
     ])
     # fmt: on
@@ -408,80 +445,49 @@ It is released under the [CC0]\
 
     # Append the functional section to the existing anatomical exerpt
     # That way we do not need to stream down the number of bold datasets
-    anat_preproc_wf.__postdesc__ = (
-        (anat_preproc_wf.__postdesc__ if hasattr(anat_preproc_wf, "__postdesc__") else "")
-        + f"""
+    anat_preproc_wf.__postdesc__ = getattr(anat_preproc_wf, '__postdesc__') or ''
+    func_pre_desc = f"""
 
 Functional data preprocessing
 
 : For each of the {len(subject_data['bold'])} BOLD runs found per subject (across all
-tasks and sessions), the following preprocessing was performed.
-"""
-    )
-
-    # calculate reference image(s) for BOLD images
-    # group all BOLD files based on same:
-    # 1) session
-    # 2) PE direction
-    # 3) total readout time
-    from niworkflows.workflows.epi.refmap import init_epi_reference_wf
-
-    bold_groupings = group_bolds_ref(
-        layout=config.execution.layout,
-        subject=subject_id,
-        sessions=[session_id],
-    )
+tasks and sessions), the following preprocessing was performed."""
 
     func_preproc_wfs = []
     has_fieldmap = bool(fmap_estimators)
-    for idx, grouping in enumerate(bold_groupings.values()):
-        bold_ref_wf = init_epi_reference_wf(
-            auto_bold_nss=True,
-            name=f"bold_reference_wf{idx}",
-            omp_nthreads=config.nipype.omp_nthreads,
-        )
-        bold_files = grouping.files
-        bold_ref_wf.inputs.inputnode.in_files = grouping.files
+    for bold_file in subject_data['bold']:
+        func_preproc_wf = init_func_preproc_wf(bold_file, spaces, has_fieldmap=has_fieldmap)
+        if func_preproc_wf is None:
+            continue
 
-        if grouping.multiecho_id is not None:
-            bold_files = [bold_files]
-        for idx, bold_file in enumerate(bold_files):
-            func_preproc_wf = init_func_preproc_wf(
-                bold_file,
-                has_fieldmap=has_fieldmap,
-                existing_derivatives=derivatives,
-            )
-            # fmt: off
-            workflow.connect([
-                (bold_ref_wf, func_preproc_wf, [
-                    ('outputnode.epi_ref_file', 'inputnode.bold_ref'),
-                    (
-                        ('outputnode.xfm_files', _select_iter_idx, idx),
-                        'inputnode.bold_ref_xfm'),
-                    (
-                        ('outputnode.n_dummy', _select_iter_idx, idx),
-                        'inputnode.n_dummy_scans'),
-                ]),
-                (anat_preproc_wf, func_preproc_wf, [
-                    ('outputnode.anat_preproc', 'inputnode.anat_preproc'),
-                    ('outputnode.anat_mask', 'inputnode.anat_mask'),
-                    ('outputnode.anat_brain', 'inputnode.anat_brain'),
-                    ('outputnode.anat_dseg', 'inputnode.anat_dseg'),
-                    ('outputnode.anat_aseg', 'inputnode.anat_aseg'),
-                    ('outputnode.anat_aparc', 'inputnode.anat_aparc'),
-                    ('outputnode.anat_tpms', 'inputnode.anat_tpms'),
-                    ('outputnode.template', 'inputnode.template'),
-                    ('outputnode.anat2std_xfm', 'inputnode.anat2std_xfm'),
-                    ('outputnode.std2anat_xfm', 'inputnode.std2anat_xfm'),
-                    # Undefined if --fs-no-reconall, but this is safe
-                    ('outputnode.subjects_dir', 'inputnode.subjects_dir'),
-                    ('outputnode.subject_id', 'inputnode.subject_id'),
-                    ('outputnode.t1w2fsnative_xfm', 'inputnode.t1w2fsnative_xfm'),
-                    ('outputnode.fsnative2t1w_xfm', 'inputnode.fsnative2t1w_xfm'),
-                ]),
-            ])
-            # fmt: on
-            func_preproc_wfs.append(func_preproc_wf)
+        func_preproc_wf.__desc__ = func_pre_desc + (getattr(func_preproc_wf, '__desc__') or '')
+        # fmt:off
+        workflow.connect([
+            (anat_preproc_wf, func_preproc_wf, [
+                ('outputnode.anat_preproc', 'inputnode.anat_preproc'),
+                ('outputnode.anat_mask', 'inputnode.anat_mask'),
+                ('outputnode.anat_brain', 'inputnode.anat_brain'),
+                ('outputnode.anat_dseg', 'inputnode.anat_dseg'),
+                ('outputnode.anat_aseg', 'inputnode.anat_aseg'),
+                ('outputnode.anat_aparc', 'inputnode.anat_aparc'),
+                ('outputnode.anat_tpms', 'inputnode.anat_tpms'),
+                ('outputnode.template', 'inputnode.template'),
+                ('outputnode.anat2std_xfm', 'inputnode.anat2std_xfm'),
+                ('outputnode.std2anat_xfm', 'inputnode.std2anat_xfm'),
+                # Undefined if --fs-no-reconall, but this is safe
+                ('outputnode.subjects_dir', 'inputnode.subjects_dir'),
+                ('outputnode.subject_id', 'inputnode.subject_id'),
+                ('outputnode.anat2fsnative_xfm', 'inputnode.t1w2fsnative_xfm'),
+                ('outputnode.fsnative2anat_xfm', 'inputnode.fsnative2t1w_xfm'),
+                ('outputnode.surfaces', 'inputnode.surfaces'),
+                ('outputnode.morphometrics', 'inputnode.morphometrics'),
+                ('outputnode.anat_ribbon', 'inputnode.anat_ribbon'),
+                ('outputnode.sphere_reg_fsLR', 'inputnode.sphere_reg_fsLR'),
+                ('outputnode.midthickness_fsLR', 'inputnode.midthickness_fsLR'),
+            ]),
+        ])
+        # fmt:on
+        func_preproc_wfs.append(func_preproc_wf)
 
     if not has_fieldmap:
         config.loggers.workflow.warning(
@@ -506,6 +512,7 @@ tasks and sessions), the following preprocessing was performed.
         subject=subject_id,
     )
     fmap_wf.__desc__ = f"""
+
 Preprocessing of B<sub>0</sub> inhomogeneity mappings
 
 : A total of {len(fmap_estimators)} fieldmaps were found available within the input
@@ -548,9 +555,6 @@ Setting-up fieldmap "{estimator.bids_id}" ({estimator.method}) with \
                 fmap_wf_inputs = getattr(fmap_wf.inputs, f"in_{estimator.bids_id}")
                 fmap_wf_inputs.in_data = [str(s.path) for s in estimator.sources]
                 fmap_wf_inputs.metadata = [s.metadata for s in estimator.sources]
-
-                flatten = fmap_wf.get_node(f"wf_{estimator.bids_id}.flatten")
-                flatten.inputs.max_trs = config.workflow.topup_max_vols
             else:
                 raise NotImplementedError(
                     "Sophisticated PEPOLAR schemes (e.g., using DWI+EPI) are unsupported."
@@ -563,8 +567,53 @@ def _prefix(subid):
     return subid if subid.startswith("sub-") else f"sub-{subid}"
 
 
-def _select_iter_idx(in_list, idx):
-    """Returns a specific index of a list/tuple"""
-    if isinstance(in_list, (tuple, list)):
-        return in_list[idx]
-    raise AttributeError(f"Input {in_list} is incompatible type: {type(in_list)}")
+def init_workflow_spaces(execution_spaces: SpatialReferences, age_months: int):
+    """
+    Create output spaces at a per-subworkflow level.
+
+    This address the case where a multi-session subject is run, and requires separate template cohorts.
+    """
+    from niworkflows.utils.spaces import Reference
+
+    from nibabies.utils.misc import cohort_by_months
+
+    spaces = deepcopy(execution_spaces)
+
+    if age_months is None:
+        raise RuntimeError("Participant age (in months) is required.")
+
+    if not spaces.references:
+        # Ensure age specific template is added if nothing is present
+        cohort = cohort_by_months("MNIInfant", age_months)
+        spaces.add(("MNIInfant", {"res": "native", "cohort": cohort}))
+
+    if not spaces.is_cached():
+        spaces.checkpoint()
+
+    # Ensure user-defined spatial references for outputs are correctly parsed.
+    # Certain options require normalization to a space not explicitly defined by users.
+    # These spaces will not be included in the final outputs.
+    if config.workflow.use_aroma:
+        # Make sure there's a normalization to FSL for AROMA to use.
+        spaces.add(Reference("MNI152NLin6Asym", {"res": "2"}))
+
+    if config.workflow.cifti_output:
+        # CIFTI grayordinates to corresponding FSL-MNI resolutions.
+        vol_res = "2" if config.workflow.cifti_output == "91k" else "1"
+        spaces.add(Reference("MNI152NLin6Asym", {"res": vol_res}))
+        # Ensure a non-native version of MNIInfant is added as a target
+        cohort = cohort_by_months("MNIInfant", age_months)
+        spaces.add(Reference("MNIInfant", {"cohort": cohort}))
+
+    return spaces
+
+
+def init_execution_spaces():
+    from niworkflows.utils.spaces import Reference, SpatialReferences
+
+    spaces = config.execution.output_spaces or SpatialReferences()
+    if not isinstance(spaces, SpatialReferences):
+        spaces = SpatialReferences(
+            [ref for s in spaces.split(" ") for ref in Reference.from_string(s)]
+        )
+    return spaces
