@@ -1,7 +1,7 @@
 # emacs: -*- mode: python; py-indent-offset: 4; indent-tabs-mode: nil -*-
 # vi: set ft=python sts=4 ts=4 sw=4 et:
 #
-# Copyright 2021 The NiPreps Developers <nipreps@gmail.com>
+# Copyright 2023 The NiPreps Developers <nipreps@gmail.com>
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -124,17 +124,13 @@ BOLD runs were slice-time corrected to {tzero:0.3g}s ({frac:g} of slice acquisit
 
     copy_xform = pe.Node(CopyXForm(), name="copy_xform", mem_gb=0.1)
 
-    workflow.connect(
-        [
-            (
-                inputnode,
-                slice_timing_correction,
-                [("bold_file", "in_file"), ("skip_vols", "ignore")],
-            ),
-            (slice_timing_correction, copy_xform, [("out_file", "in_file")]),
-            (inputnode, copy_xform, [("bold_file", "hdr_file")]),
-            (copy_xform, outputnode, [("out_file", "stc_file")]),
-        ]
-    )
-
+    # fmt:off
+    workflow.connect([
+        (inputnode, slice_timing_correction, [("bold_file", "in_file"),
+                                              ("skip_vols", "ignore")]),
+        (slice_timing_correction, copy_xform, [("out_file", "in_file")]),
+        (inputnode, copy_xform, [("bold_file", "hdr_file")]),
+        (copy_xform, outputnode, [("out_file", "stc_file")]),
+    ])
+    # fmt:on
     return workflow
